package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoDestruir
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoInvestigar
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoReparar
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.Opcion
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoIgnorar
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.ObjetoDeValor

class ReliquiaFactory : EventoFactory {
    private val reliquiasMap = mapOf(
        "Encuentras un vasito de barro cocido en las tumbas de la necrópolis de Las Ruedas." to Pair(
            R.drawable.asi_exploradores_vasito, listOf(
                Opcion(
                    descripcion = "Investigar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoInvestigar(
                        70,
                        "Al observar los grabados y la técnica de cocción, descubres que el vaso es una pieza de los vacceos, una cultura prerromana que habitó el valle del Duero.",
                        1
                    )
                ), Opcion(
                    descripcion = "Reparar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoReparar(
                        65,
                        "Tras aplicar con cuidado una mezcla de resina especial, consigues unir las piezas del vasito, restaurando su forma original. Has preservado una pieza valiosa de la historia vaccea.",
                        "Al intentar reparar el vasito, accidentalmente lo rompes aún más. Ahora, no solo está dañado, sino que se ha perdido una pieza esencial que no puede ser reconstruida.",
                        1,
                        ObjetoDeValor(
                            nombre = "Vasito de barro cocido",
                            descripcion = "Objeto de gran valor. Se convierte en 20 monedas si consigues completar la partida.",
                            coste = 20,
                            imagen = R.drawable.asi_exploradores_vasito
                        )

                    )
                ), Opcion(
                    descripcion = "Destruir",
                    atributoRequerida = Atributo.Ataque,
                    EfectoDestruir(90)

                ), Opcion(
                    descripcion = "Ignorar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)

                )
            )
        ),
        "Encuentras una jabonera de cerámica con un símbolo en ella." to Pair(
            R.drawable.asi_exploradores_jabonera, listOf(
                Opcion(
                    descripcion = "Investigar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoInvestigar(
                        60,
                        "Tras examinar minuciosamente la jabonera, descubres que el símbolo representado es un lobo, figura clave en la mitología vaccea, simbolizando miedo y veneración.",
                        1
                    )

                ), Opcion(
                    descripcion = "Reparar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoReparar(
                        55,
                        "Con un toque experto, las partes rotas encajan perfectamente. Has dado nueva vida a una jabonera vaccea, permitiéndote estudiar sus detalles con mayor precisión.",
                        "El intento de reparación es un desastre: la jabonera se rompe aún más, y el símbolo se desintegra. Ahora, solo quedan fragmentos de lo que antes fue una obra de arte.",
                        1,
                        ObjetoDeValor(
                            nombre = "Jabonera de cerámica",
                            descripcion = "Objeto de gran valor. Se convierte en 20 monedas si consigues completar la partida.",
                            coste = 20,
                            imagen = R.drawable.asi_exploradores_jabonera
                        )
                    )

                ), Opcion(
                    descripcion = "Destruir",
                    atributoRequerida = Atributo.Ataque,
                    EfectoDestruir(80)
                ), Opcion(
                    descripcion = "Ignorar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
        "Encuentras una copa negra cubierta de inscripciones." to Pair(
            R.drawable.asi_minijuego_3_copa_negra_brunida, listOf(
                Opcion(
                    descripcion = "Investigar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoInvestigar(
                        60,
                        "Tras examinar minuciosamente la copa, encuentras patrones geométricos que podrían relacionarse con antiguos ritos funerarios o de sacrificio. ",
                        1
                    )

                ), Opcion(
                    descripcion = "Reparar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoReparar(
                        55,
                        "Reparas la copa con éxito y descubres un patrón o símbolo oculto bajo las grietas, aportando nueva información sobre su origen o uso.",
                        "El trabajo improvisado falla, y la copa queda aún más dañada o inutilizable, reduciendo su valor cultural o práctico.",
                        1,
                        ObjetoDeValor(
                            nombre = "Copa negra con inscripciones",
                            descripcion = "Objeto de gran valor. Se convierte en 20 monedas si consigues completar la partida.",
                            coste = 20,
                            imagen = R.drawable.asi_minijuego_3_copa_negra_brunida
                        )
                    )

                ), Opcion(
                    descripcion = "Destruir",
                    atributoRequerida = Atributo.Ataque,
                    EfectoDestruir(80)
                ), Opcion(
                    descripcion = "Ignorar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
        "Encuentra una tapadera de cerámica con una figura zoomorfa tallada en su superficie. Mmuestra el detalle de un animal: un lobo" to Pair(
            R.drawable.asi_minijuego_3_tapadera_zoomorfo, listOf(
                Opcion(
                    descripcion = "Investigar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoInvestigar(
                        60,
                        "Descubres que la tapadera representa a un animal sagrado en la cultura vaccea, podría estar relacionado con rituales de protección o iniciación. ",
                        1
                    )

                ), Opcion(
                    descripcion = "Reparar",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoReparar(
                        55,
                        "Logras repararla cuidadosamente, las marcas en la cerámica se conecten con un ritual funerario o de veneración, lo que incrementa el valor histórico de la pieza.",
                        "Intentas repararla, pero sin las herramientas adecuadas o la técnica correcta, la tapadera queda más dañada.",
                        1,
                        ObjetoDeValor(
                            nombre = "Tapadera de cerámica zoomorfa",
                            descripcion = "Objeto de gran valor. Se convierte en 20 monedas si consigues completar la partida.",
                            coste = 20,
                            imagen = R.drawable.asi_minijuego_3_tapadera_zoomorfo
                        )
                    )

                ), Opcion(
                    descripcion = "Destruir",
                    atributoRequerida = Atributo.Ataque,
                    EfectoDestruir(80)
                ), Opcion(
                    descripcion = "Ignorar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
    )

    override fun generarDescripcion(): String {
        return reliquiasMap.keys.random()
    }

    override fun generarOpciones(descripcion: String): MutableList<Opcion> {
        return reliquiasMap[descripcion]?.second?.shuffled()?.take(4)?.toMutableList()
            ?: mutableListOf()
    }

    override fun generarImagen(descripcion: String): Int {
        return reliquiasMap[descripcion]?.first!!
    }

    override fun generarEvento(): EventoReliquia {
        val descripcion = generarDescripcion()
        return EventoReliquia(
            descripcion = descripcion,
            dificultad = 0,
            opciones = generarOpciones(descripcion),
            imagen = generarImagen(descripcion),
            resistenciaMax = (2..8).random(),
            resistenciaActual = 0,
            finalizado = false
        )
    }
}
