package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.Opcion
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoIgnorar
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoRiesgo
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.ObjetoEquipable
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoEquipableAumento


class RiesgoFactory : EventoFactory {
    private val riesgoMap = mapOf(
        "En un claro junto al río Duero, encuentras un viejo cofre de madera decorado con espigas de trigo y motivos solares. " to Pair(
            R.drawable.cofre_open_animation, listOf(
                Opcion(
                    descripcion = "Abrir el cofre",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoRiesgo(
                        ObjetoEquipable(
                            nombre = "Espada de Hierro",
                            descripcion = "Una espada de hierro sencilla y confiable perfecta para un aventurero novato. Otorga 1 de daño extra",
                            efecto = EfectoEquipableAumento(1, Atributo.Ataque),
                            imagen = R.drawable.asi_objeto_espadadefault
                        ),
                        dmgRiesgo = 2,
                        mensajeRecomensa = "Dentro del cofre encuentras una espada de hierro vaccea. Está en buen estado y tiene un brillo sobrenatural",
                        mensajeRiesgo = "Una nube de polvo tóxico se libera, causandote 2 de daño",
                    )
                ), Opcion(
                    descripcion = "Ignorar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
    )

    override fun generarDescripcion(): String {
        return riesgoMap.keys.random()
    }

    override fun generarOpciones(descripcion: String): MutableList<Opcion> {
        return riesgoMap[descripcion]?.second?.toMutableList()
            ?: mutableListOf()
    }

    override fun generarImagen(descripcion: String): Int {
        return riesgoMap[descripcion]?.first!!
    }

    override fun generarEvento(): EventoRiesgo {
        val descripcion = generarDescripcion()
        return EventoRiesgo(
            descripcion = descripcion,
            dificultad = 0,
            opciones = generarOpciones(descripcion),
            imagen = generarImagen(descripcion),
            resistenciaMax = (2..8).random(),
            finalizado = false
        )
    }
}