package com.example.ellegadodepintia.minijuegoReparacion

import android.app.Activity
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Rect
import android.graphics.drawable.AnimationDrawable
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.widget.ImageView
import android.widget.RelativeLayout
import androidx.appcompat.app.AppCompatActivity
import com.example.ellegadodepintia.R
import android.os.Handler
import android.os.Looper
import android.view.animation.LinearInterpolator
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState
import kotlin.random.Random

class MinijuegoReparacion : AppCompatActivity() {

    private lateinit var fragmentos: List<Fragmento>
    private val filas = 5
    private val columnas = 5
    private var clics = 5
    private var intervaloDeAngulo = Pair(0f, 0f)
    val handler = Handler()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_minijuego_reparacion)
        var imagenOriginal: Bitmap? = null
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        if(flag){
             imagenOriginal = cargarImagen(R.drawable.asi_exploradores_jabonera)
        }else{
             imagenOriginal = cargarImagen(GameState.eventoActual.imagen)
        }

        fragmentos = dividirImagen(imagenOriginal, filas, columnas)

        val relativeLayout = findViewById<RelativeLayout>(R.id.relativeLayoutFragmentos)
        agregarFragmentosAlRelativeLayout(relativeLayout)

        val paloImageView: ImageView = findViewById(R.id.QTEPalo)
        val ruleImageView: ImageView = findViewById(R.id.QTERule)
        iniciarQTE(paloImageView, ruleImageView)

    }

    private fun cargarImagen(idRecurso: Int): Bitmap {
        return BitmapFactory.decodeResource(resources, idRecurso)
    }

    private fun dividirImagen(imagen: Bitmap, filas: Int, columnas: Int): List<Fragmento> {
        val fragmentos = mutableListOf<Fragmento>()
        val ancho = imagen.width
        val alto = imagen.height
        val tamanoFragmento = minOf(ancho, alto) / columnas

        for (i in 0 until filas) {
            for (j in 0 until columnas) {
                val xInicio = j * tamanoFragmento
                val yInicio = i * tamanoFragmento
                val rectangulo = Rect(xInicio, yInicio, xInicio + tamanoFragmento, yInicio + tamanoFragmento)
                val fragmento = Bitmap.createBitmap(imagen, rectangulo.left, rectangulo.top, rectangulo.width(), rectangulo.height())
                fragmentos.add(Fragmento(fragmento, i, j))
            }
        }

        return fragmentos
    }

    private fun agregarFragmentosAlRelativeLayout(relativeLayout: RelativeLayout) {
        val mesaHeightMin = 1700
        val mesaHeightMax = 1900

        for (fragmento in fragmentos) {
            val imageView = ImageView(this)
            imageView.setImageBitmap(fragmento.bitmap)
            val params = RelativeLayout.LayoutParams(120, 120)

            params.leftMargin = Random.nextInt(250, 750)
            params.topMargin = Random.nextInt(mesaHeightMin, mesaHeightMax)
            imageView.layoutParams = params

            imageView.rotation = Random.nextFloat() * 90 - 45
            fragmento.xPos = params.leftMargin
            fragmento.yPos = params.topMargin
            fragmento.rotation = imageView.rotation

            relativeLayout.addView(imageView)
        }
    }

    private fun reconstruirImagenEnPartes(relativeLayout: RelativeLayout) {
        if (clics == -1) return
        mostrarNubesDePolvo(relativeLayout,40)
        mostrarMartilloGolpe(relativeLayout)
        for ((index, fragmento) in fragmentos.withIndex()) {
            val imageView = relativeLayout.getChildAt(index) as? ImageView ?: continue
            val margenIzquierdaFinal = fragmento.columna * 120 + 250
            val margenArribaFinal = fragmento.fila * 120 + 1300
            val precisionRandom = Random.nextFloat() * 0.4f + 0.6f
            val precision = 1f - (clics * 0.20f) * precisionRandom

            val margenIzquierda = fragmento.xPos + (margenIzquierdaFinal - fragmento.xPos) * precision
            val margenArriba = fragmento.yPos + (margenArribaFinal - fragmento.yPos) * precision
            val rotacion = fragmento.rotation * (1 - precision)

            val params = RelativeLayout.LayoutParams(120, 120)
            params.leftMargin = margenIzquierda.toInt()
            params.topMargin = margenArriba.toInt()
            imageView.layoutParams = params

            imageView.animate()
                .x(margenIzquierda)
                .y(margenArriba)
                .rotation(rotacion)
                .setDuration(500)
                .start()
        }
    }

    data class Fragmento(val bitmap: Bitmap, val fila: Int, val columna: Int, var xPos: Int = 0, var yPos: Int = 0, var rotation: Float = 0f)

    private fun mostrarQTE(paloImageView: ImageView, ruleImageView: ImageView) {
        val relativeLayout = findViewById<RelativeLayout>(R.id.relativeLayoutFragmentos)
        var para = false
        val randomX = Random.nextFloat() * (1050 - 350)
        val randomY = Random.nextFloat() * (1400 - 250)
        val randomR = 150 + Random.nextFloat() * (360 - 150)
        paloImageView.x = randomX + 75
        paloImageView.y = randomY + 130
        ruleImageView.x = randomX
        ruleImageView.y = randomY
        ruleImageView.rotation = randomR

        fadeIn(paloImageView, ruleImageView)
        intervaloDeAngulo = Pair((270f + randomR) % 360, (330f + randomR) % 360).let {
            if (it.first > it.second) it.second to it.first else it
        }

        paloImageView.rotation = Random.nextFloat() * 180
        val animation = paloImageView.animate()
            .rotationBy(1080f)
            .setInterpolator(LinearInterpolator())
            .setDuration(5000)

        animation.start()
        relativeLayout.setOnTouchListener { _, _ ->
            if (paloImageView.visibility == ImageView.VISIBLE && !para) {
                animation.cancel()
                para = true
                verificarQTE(paloImageView.rotation)
            }
            true
        }
    }

    private fun verificarQTE(anguloActual: Float) {
        val relativeLayout = findViewById<RelativeLayout>(R.id.relativeLayoutFragmentos)
        if (anguloActual % 360 in intervaloDeAngulo.first..intervaloDeAngulo.second) {
            if (clics > 0) {
                clics--
                reconstruirImagenEnPartes(relativeLayout)
            }

        } else {
            if (clics < 5) {
                clics++
                reconstruirImagenEnPartes(relativeLayout)
            }
        }
    }

    private fun iniciarQTE(paloImageView: ImageView, ruleImageView: ImageView) {
        val handler = Handler(Looper.getMainLooper())
        val runnable = object : Runnable {
            override fun run() {
                if (clics == 0) {
                    handler.removeCallbacks(this)
                    val relativeLayout = findViewById<RelativeLayout>(R.id.relativeLayoutFragmentos)
                    mostrarNubesDePolvo(relativeLayout,100)

                    val resultIntent = Intent()
                    resultIntent.putExtra("resultado", "Reparación completada")
                    setResult(Activity.RESULT_OK, resultIntent)
                    finish()
                    return
                }
                mostrarQTE(paloImageView, ruleImageView)
                handler.postDelayed({ fadeOut(paloImageView, ruleImageView) }, 2500)
                handler.postDelayed(this, 3000)
            }
        }
        handler.postDelayed(runnable, 100)
    }

    fun fadeIn(paloImageView: ImageView, ruleImageView: ImageView) {
        paloImageView.alpha = 0f
        ruleImageView.alpha = 0f
        paloImageView.visibility = ImageView.VISIBLE
        ruleImageView.visibility = ImageView.VISIBLE

        paloImageView.animate().alpha(1f).setDuration(500).start()
        ruleImageView.animate().alpha(1f).setDuration(500).start()
    }

    fun fadeOut(paloImageView: ImageView, ruleImageView: ImageView) {
        paloImageView.animate()
            .alpha(0f)
            .setDuration(500)
            .withEndAction { paloImageView.visibility = ImageView.GONE }
            .start()
        ruleImageView.animate()
            .alpha(0f)
            .setDuration(500)
            .withEndAction { ruleImageView.visibility = ImageView.GONE }
            .start()
    }

    private fun mostrarNubesDePolvo(relativeLayout: RelativeLayout, cantidad: Int) {
        for (i in 0 until cantidad) {
            val nubeView = ImageView(this)
            nubeView.setBackgroundResource(R.drawable.effect_reparar_animation)
            val tam = Random.nextInt(150, 250)
            val params = RelativeLayout.LayoutParams(tam, tam)
            params.leftMargin = Random.nextInt(100, 800)
            params.topMargin = Random.nextInt(1300, 1900)
            nubeView.layoutParams = params
            nubeView.rotation = Random.nextFloat()* 360
            relativeLayout.addView(nubeView)
            val delay = Random.nextLong(0, 300)
            handler.postDelayed({
                (nubeView.background as? AnimationDrawable)?.start()
                nubeView.animate()
                    .translationYBy(-Random.nextFloat()* 200)
                    .alpha(0f)
                    .setDuration(1000L)
                    .withEndAction { relativeLayout.removeView(nubeView) }
                    .start()
            }, delay)
        }
    }

    private fun mostrarMartilloGolpe(relativeLayout: RelativeLayout) {
        val martilloView = ImageView(this)
        martilloView.setImageResource(R.drawable.asi_reparar_martillo)
        val tamMartillo = Random.nextInt(300, 500)
        val params = RelativeLayout.LayoutParams(tamMartillo, tamMartillo)
        params.leftMargin = Random.nextInt(100, 700)
        params.topMargin = Random.nextInt(1300, 1900)
        martilloView.rotation = 50f
        martilloView.layoutParams = params
        relativeLayout.addView(martilloView)
        martilloView.animate()
            .rotationBy(30f)
            .setDuration(300L)
            .withEndAction {
                martilloView.animate()
                    .alpha(0f)
                    .setDuration(300L)
                    .withEndAction { relativeLayout.removeView(martilloView) }
                    .start()
            }
            .start()
    }



}
